@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2015 jointly by the following organizations:
+ * Copyright (C) 2013-2018 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
@@ -156,20 +156,11 @@ InitialAssignment::getSymbol () const
 }
 
 
-/** @cond doxygenLibsbmlInternal */
-/**
- * @return the string of symbol attribute of this object.
- * 
- * @note this function is an alias for getSymbol()
- * 
- * @see getSymbol()
- */
 const std::string& 
 InitialAssignment::getId() const
 {
   return getSymbol();
 }
-/** @endcond */
 
 
 /*
@@ -183,7 +174,7 @@ InitialAssignment::getMath () const
 
 
 /*
- * @return true if the symbol of this InitialAssignment is set,
+ * @return @c true if the symbol of this InitialAssignment is set,
  * false otherwise.
  */
 bool
@@ -194,7 +185,7 @@ InitialAssignment::isSetSymbol () const
 
 
 /*
- * @return true if the math for this InitialAssignment is set,
+ * @return @c true if the math for this InitialAssignment is set,
  * false otherwise.
  */
 bool
@@ -205,7 +196,7 @@ InitialAssignment::isSetMath () const
 
 
 /*
- * Sets the symbol of this InitialAssignment to a copy of sid.
+ * Sets the symbol of this InitialAssignment to a copy of @p sid.
  */
 int
 InitialAssignment::setSymbol (const std::string& sid)
@@ -308,10 +299,10 @@ InitialAssignment::getDerivedUnitDefinition()
       m->populateListFormulaUnitsData();
     }
     
-    if (m->getFormulaUnitsData(getId(), getTypeCode()) != NULL)
+    FormulaUnitsData *fud = m->getFormulaUnitsData(getId(), getTypeCode());
+    if (fud != NULL)
     {
-      return m->getFormulaUnitsData(getId(), getTypeCode())
-                                             ->getUnitDefinition();
+      return fud->getUnitDefinition();
     }
     else
     {
@@ -336,6 +327,7 @@ InitialAssignment::getDerivedUnitDefinition() const
 }
 
 
+/** @cond doxygenLibsbmlInternal */
 /*
  * Predicate returning @c true if 
  * the math expression of this InitialAssignment contains
@@ -377,10 +369,10 @@ InitialAssignment::containsUndeclaredUnits()
       m->populateListFormulaUnitsData();
     }
     
-    if (m->getFormulaUnitsData(getId(), getTypeCode()) != NULL)
+    FormulaUnitsData *fud = m->getFormulaUnitsData(getId(), getTypeCode());
+    if (fud != NULL)
     {
-      return m->getFormulaUnitsData(getId(), getTypeCode())
-      ->getContainsUndeclaredUnits();
+      return fud->getContainsUndeclaredUnits();
     }
     else
     {
@@ -392,13 +384,16 @@ InitialAssignment::containsUndeclaredUnits()
     return false;
   }
 }
+/** @endcond */
 
 
+/** @cond doxygenLibsbmlInternal */
 bool 
 InitialAssignment::containsUndeclaredUnits() const
 {
   return const_cast<InitialAssignment *> (this)->containsUndeclaredUnits();
 }
+/** @endcond */
 
 
 /*
@@ -445,17 +440,319 @@ InitialAssignment::hasRequiredElements() const
   bool allPresent = true;
 
   /* required attributes for initialAssignment: math */
+  /* l3v2 removed that requirement */
 
-  if (!isSetMath())
-    allPresent = false;
+  if ((getLevel() < 3 ) || (getLevel() == 3 && getVersion() == 1))
+  {
+    if (!isSetMath())
+      allPresent = false;
+  }
 
   return allPresent;
 }
 
 
+
+
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Gets the value of the "attributeName" attribute of this InitialAssignment.
+ */
+int
+InitialAssignment::getAttribute(const std::string& attributeName,
+                                bool& value) const
+{
+  int return_value = SBase::getAttribute(attributeName, value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Gets the value of the "attributeName" attribute of this InitialAssignment.
+ */
+int
+InitialAssignment::getAttribute(const std::string& attributeName,
+                                int& value) const
+{
+  int return_value = SBase::getAttribute(attributeName, value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Gets the value of the "attributeName" attribute of this InitialAssignment.
+ */
+int
+InitialAssignment::getAttribute(const std::string& attributeName,
+                                double& value) const
+{
+  int return_value = SBase::getAttribute(attributeName, value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Gets the value of the "attributeName" attribute of this InitialAssignment.
+ */
+int
+InitialAssignment::getAttribute(const std::string& attributeName,
+                                unsigned int& value) const
+{
+  int return_value = SBase::getAttribute(attributeName, value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Gets the value of the "attributeName" attribute of this InitialAssignment.
+ */
+int
+InitialAssignment::getAttribute(const std::string& attributeName,
+                                std::string& value) const
+{
+  int return_value = SBase::getAttribute(attributeName, value);
+
+  if (return_value == LIBSBML_OPERATION_SUCCESS)
+  {
+    return return_value;
+  }
+
+  if (attributeName == "symbol")
+  {
+    value = getSymbol();
+    return_value = LIBSBML_OPERATION_SUCCESS;
+  }
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Gets the value of the "attributeName" attribute of this InitialAssignment.
+ */
+//int
+//InitialAssignment::getAttribute(const std::string& attributeName,
+//                                const char* value) const
+//{
+//  int return_value = SBase::getAttribute(attributeName, value);
+//
+//  if (return_value == LIBSBML_OPERATION_SUCCESS)
+//  {
+//    return return_value;
+//  }
+//
+//  if (attributeName == "symbol")
+//  {
+//    value = getSymbol().c_str();
+//    return_value = LIBSBML_OPERATION_SUCCESS;
+//  }
+//
+//  return return_value;
+//}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Predicate returning @c true if this InitialAssignment's attribute
+ * "attributeName" is set.
+ */
+bool
+InitialAssignment::isSetAttribute(const std::string& attributeName) const
+{
+  bool value = SBase::isSetAttribute(attributeName);
+
+  if (attributeName == "symbol")
+  {
+    value = isSetSymbol();
+  }
+
+  return value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Sets the value of the "attributeName" attribute of this InitialAssignment.
+ */
+int
+InitialAssignment::setAttribute(const std::string& attributeName, bool value)
+{
+  int return_value = SBase::setAttribute(attributeName, value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Sets the value of the "attributeName" attribute of this InitialAssignment.
+ */
+int
+InitialAssignment::setAttribute(const std::string& attributeName, int value)
+{
+  int return_value = SBase::setAttribute(attributeName, value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Sets the value of the "attributeName" attribute of this InitialAssignment.
+ */
+int
+InitialAssignment::setAttribute(const std::string& attributeName,
+                                double value)
+{
+  int return_value = SBase::setAttribute(attributeName, value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Sets the value of the "attributeName" attribute of this InitialAssignment.
+ */
+int
+InitialAssignment::setAttribute(const std::string& attributeName,
+                                unsigned int value)
+{
+  int return_value = SBase::setAttribute(attributeName, value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Sets the value of the "attributeName" attribute of this InitialAssignment.
+ */
+int
+InitialAssignment::setAttribute(const std::string& attributeName,
+                                const std::string& value)
+{
+  int return_value = SBase::setAttribute(attributeName, value);
+
+  if (attributeName == "symbol")
+  {
+    return_value = setSymbol(value);
+  }
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Sets the value of the "attributeName" attribute of this InitialAssignment.
+ */
+//int
+//InitialAssignment::setAttribute(const std::string& attributeName,
+//                                const char* value)
+//{
+//  int return_value = SBase::setAttribute(attributeName, value);
+//
+//  if (attributeName == "symbol")
+//  {
+//    return_value = setSymbol(value);
+//  }
+//
+//  return return_value;
+//}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Unsets the value of the "attributeName" attribute of this InitialAssignment.
+ */
+int
+InitialAssignment::unsetAttribute(const std::string& attributeName)
+{
+  int value = SBase::unsetAttribute(attributeName);
+
+  if (attributeName == "symbol")
+  {
+    value = unsetSymbol();
+  }
+
+  return value;
+}
+
+/** @endcond */
+
+
+
+
 void
 InitialAssignment::renameSIdRefs(const std::string& oldid, const std::string& newid)
 {
+  SBase::renameSIdRefs(oldid, newid);
   if (mSymbol == oldid) {
     setSymbol(newid);
   }
@@ -467,6 +764,7 @@ InitialAssignment::renameSIdRefs(const std::string& oldid, const std::string& ne
 void 
 InitialAssignment::renameUnitSIdRefs(const std::string& oldid, const std::string& newid)
 {
+  SBase::renameUnitSIdRefs(oldid, newid);
   if (isSetMath()) {
     mMath->renameUnitSIdRefs(oldid, newid);
   }
@@ -517,7 +815,7 @@ InitialAssignment::multiplyAssignmentsToSIdByFunction(const std::string& id, con
 /** @cond doxygenLibsbmlInternal */
 /*
  * Subclasses should override this method to write out their contained
- * SBML objects as XML elements.  Be sure to call your parents
+ * SBML objects as XML elements.  Be sure to call your parent's
  * implementation of this method as well.
  */
 void
@@ -540,7 +838,7 @@ InitialAssignment::writeElements (XMLOutputStream& stream) const
  * Subclasses should override this method to read (and store) XHTML,
  * MathML, etc. directly from the XMLInputStream.
  *
- * @return true if the subclass read from the stream, false otherwise.
+ * @return @c true if the subclass read from the stream, false otherwise.
  */
 bool
 InitialAssignment::readOtherXML (XMLInputStream& stream)
@@ -554,7 +852,7 @@ InitialAssignment::readOtherXML (XMLInputStream& stream)
     if (getLevel() == 1) 
     {
       logError(NotSchemaConformant, getLevel(), getVersion(),
-	       "SBML Level 1 does not support MathML.");
+         "SBML Level 1 does not support MathML.");
       delete mMath;
       return false;
     }
@@ -564,8 +862,8 @@ InitialAssignment::readOtherXML (XMLInputStream& stream)
       if (getLevel() < 3) 
       {
         logError(NotSchemaConformant, getLevel(), getVersion(),
-	        "Only one <math> element is permitted inside a "
-	        "particular containing element.");
+          "Only one <math> element is permitted inside a "
+          "particular containing element.");
       }
       else
       {
@@ -624,7 +922,7 @@ InitialAssignment::addExpectedAttributes(ExpectedAttributes& attributes)
 /*
  * Subclasses should override this method to read values from the given
  * XMLAttributes set into their specific fields.  Be sure to call your
- * parents implementation of this method as well.
+ * parent's implementation of this method as well.
  */
 void
 InitialAssignment::readAttributes (const XMLAttributes& attributes,
@@ -639,13 +937,13 @@ InitialAssignment::readAttributes (const XMLAttributes& attributes,
   {
   case 1:
     logError(NotSchemaConformant, level, version,
-	      "InitialAssignment is not a valid component for this level/version.");
+        "InitialAssignment is not a valid component for this level/version.");
     break;
   case 2:
     if (version == 1)
     {
       logError(NotSchemaConformant, level, version,
-	        "InitialAssignment is not a valid component for this level/version.");
+          "InitialAssignment is not a valid component for this level/version.");
     }
     else
     {
@@ -665,7 +963,7 @@ InitialAssignment::readAttributes (const XMLAttributes& attributes,
 /*
  * Subclasses should override this method to read values from the given
  * XMLAttributes set into their specific fields.  Be sure to call your
- * parents implementation of this method as well.
+ * parent's implementation of this method as well.
  */
 void
 InitialAssignment::readL2Attributes (const XMLAttributes& attributes)
@@ -690,7 +988,7 @@ InitialAssignment::readL2Attributes (const XMLAttributes& attributes)
   //
   if (version == 2)
     mSBOTerm = SBO::readTerm(attributes, this->getErrorLog(), level, version,
-				getLine(), getColumn());
+        getLine(), getColumn());
 }
 /** @endcond */
 
@@ -699,7 +997,7 @@ InitialAssignment::readL2Attributes (const XMLAttributes& attributes)
 /*
  * Subclasses should override this method to read values from the given
  * XMLAttributes set into their specific fields.  Be sure to call your
- * parents implementation of this method as well.
+ * parent's implementation of this method as well.
  */
 void
 InitialAssignment::readL3Attributes (const XMLAttributes& attributes)
@@ -731,7 +1029,7 @@ InitialAssignment::readL3Attributes (const XMLAttributes& attributes)
 /** @cond doxygenLibsbmlInternal */
 /*
  * Subclasses should override this method to write their XML attributes
- * to the XMLOutputStream.  Be sure to call your parents implementation
+ * to the XMLOutputStream.  Be sure to call your parent's implementation
  * of this method as well.
  */
 void
@@ -837,11 +1135,11 @@ ListOfInitialAssignments::get(unsigned int n) const
  */
 struct IdEqIA : public unary_function<SBase*, bool>
 {
-  const string& id;
+  const string& mId;
 
-  IdEqIA (const string& id) : id(id) { }
+  IdEqIA (const string& id) : mId(id) { }
   bool operator() (SBase* sb) 
-       { return static_cast <InitialAssignment *> (sb)->getId() == id; }
+       { return static_cast <InitialAssignment *> (sb)->getId() == mId; }
 };
 
 
